@@ -62,7 +62,7 @@ sub image_type {
     ($t) = image_getvalue($image, 'format'); }
   # Note that Image::Magick (sometimes) returns "descriptive" types
   # (but I can't find a list anywhere!)
-  $t = 'eps' if $t =~ /postscript/i;
+  $t = 'eps' if $t && $t =~ /postscript/i;
   return (defined $t ? lc($t) : undef); }
 
 sub image_size {
@@ -275,6 +275,13 @@ sub image_graphicx_complex {
   my $dppt       = $properties{dppt}       || $DOTS_PER_POINT;
   my $background = $properties{background} || $BACKGROUND;
   my $image = image_read($source, antialias => 1) or return;
+  # Get some defaults from the read-in image.
+  my ($imagedpi) = image_getvalue($image, 'x-resolution');
+  # image_setvalue($image,debug=>'exception');
+  my ($bg) = image_getvalue($image, 'transparent-color');
+  $background = "rgba($bg)" if $bg;    # Use background from image, if any.
+  my ($hasalpha) = image_getvalue($image, 'matte');
+
   image_internalop($image, 'Trim') or return if $properties{autocrop};
   my $orig_ncolors = image_getvalue($image, 'colors');
   return unless $orig_ncolors;
@@ -306,7 +313,7 @@ sub image_graphicx_complex {
     NoteProgressDetailed(" [reloading to desired size $w x $h (density = $dx x $dy)]");
     $image = image_read($source, antialias => 1, density => $dx . 'x' . $dy) or return;
     image_internalop($image, 'Trim') or return if $properties{autocrop};
-    image_setvalue($image, colorspace => 'RGB') or return;
+    #    image_setvalue($image, colorspace => 'RGB') or return;
     image_internalop($image, 'Scale', geometry => int(100 / $X) . "%") or return;    # Now downscale.
     ($w, $h) = image_getvalue($image, 'width', 'height');
     return unless $w && $h; }
@@ -327,7 +334,7 @@ sub image_graphicx_complex {
       $notes .= " scale-to $w x $h";
       image_internalop($image, 'Scale', width => $w, height => $h) or return; }
     elsif ($op eq 'rotate') {
-      image_internalop($image, 'Rotate', degrees => -$a1, color => $background) or return;
+      image_internalop($image, 'Rotate', degrees => -$a1, background => $background) or return;
       ($w, $h) = image_getvalue($image, 'width', 'height');
       return unless $w && $h;
       $notes .= " rotate $a1 to $w x $h"; }
@@ -338,13 +345,15 @@ sub image_graphicx_complex {
     # but ImageMagick's coordinates are relative to upper left.
     elsif (($op eq 'trim') || ($op eq 'clip')) {
       my ($x0, $y0, $ww, $hh);
+      # Use the image's dpi for trim & clip!
+      my $idppt = (defined $imagedpi ? ($imagedpi / 72.0) : $dppt);
       if ($op eq 'trim') {    # Amount to trim: a1=left, a2=bottom, a3=right, a4=top
-        ($x0, $y0, $ww, $hh) = (floor($a1 * $dppt), floor($a4 * $dppt),
-          ceil($w - ($a1 + $a3) * $dppt), ceil($h - ($a4 + $a2) * $dppt));
+        ($x0, $y0, $ww, $hh) = (floor($a1 * $idppt), floor($a4 * $idppt),
+          ceil($w - ($a1 + $a3) * $idppt), ceil($h - ($a4 + $a2) * $idppt));
         $notes .= " trim to $ww x $hh @ $x0,$y0"; }
       else {                  # BBox: a1=left, a2=bottom, a3=right, a4=top
-        ($x0, $y0, $ww, $hh) = (floor($a1 * $dppt), floor($h - $a4 * $dppt),
-          ceil(($a3 - $a1) * $dppt), ceil(($a4 - $a2) * $dppt));
+        ($x0, $y0, $ww, $hh) = (floor($a1 * $idppt), floor($h - $a4 * $idppt),
+          ceil(($a3 - $a1) * $idppt), ceil(($a4 - $a2) * $idppt));
         $notes .= " clip to $ww x $hh @ $x0,$y0"; }
 
       if (($x0 > 0) || ($y0 > 0) || ($x0 + $ww < $w) || ($y0 + $hh < $h)) {
@@ -356,12 +365,17 @@ sub image_graphicx_complex {
         $h = min($hh + $y0, $h - $y0p);
         $notes .= " crop $w x $h @ $x0p,$y0p"; }
       # No direct `padding' operation in ImageMagick
-      my $nimage = image_read("xc:$background", size => "$ww x $hh") or return;
-      image_internalop($nimage, 'Composite', image => $image, compose => 'over', x => -$x0, y => -$y0) or return;
-      $image = $nimage;
-      ($w, $h) = ($ww, $hh);
+      # BUT, trim & clip really shouldn't need to pad???
+      # And besides, this composition seems to mangle the colormap (depending on background)
+      # my $nimage = image_read("xc:$background", size => "$ww x $hh") or return;
+      # image_internalop($nimage, 'Composite', image => $image, compose => 'over',
+      #                  x => -$x0, y => -$y0) or return;
+      # $notes .= " compose to $ww x $hh at $x0,$y0";
+      # $image = $nimage;
+      # ($w, $h) = ($ww, $hh);
     } }
-  if (my $trans = $properties{transparent}) {
+
+  if ($properties{transparent} && !$hasalpha) {
     $notes .= " transparent=$background";
     image_internalop($image, 'Transparent', $background) or return; }
 
